@@ -47,17 +47,17 @@
 #include <xmlrpc-c/server_abyss.h>
 #include <xmlrpc-c/util.h>
 
-/** Time select() will block waiting for a file descriptor to become ready */
+/** Time select() will block waiting for a file descriptor to become ready. */
 #define DEFAULT_SELECT_TIMEOUT  10000000
 
-enum flow_state
+enum flow_state_t
 {
 	/* SOURCE */
 	GRIND_WAIT_CONNECT = 0,
 	/* DESTINATION */
 	GRIND_WAIT_ACCEPT,
 	/* RUN */
-	GRIND
+	GRIND,
 };
 
 struct flow_source_settings
@@ -74,8 +74,8 @@ struct flow
 {
 	int id;
 
-	enum flow_state state;
-	enum flow_endpoint_type endpoint;
+	enum flow_state_t state;
+	enum endpoint_t endpoint;
 
 	int fd;
 	int listenfd_data;
@@ -97,11 +97,11 @@ struct flow
 	char *read_block;
 	char *write_block;
 
-	unsigned int current_write_block_size;
-	unsigned int current_read_block_size;
+	unsigned current_write_block_size;
+	unsigned current_read_block_size;
 
-	unsigned int current_block_bytes_read;
-	unsigned int current_block_bytes_written;
+	unsigned current_block_bytes_read;
+	unsigned current_block_bytes_written;
 
 	unsigned short requested_server_test_port;
 
@@ -113,9 +113,9 @@ struct flow
 
 	int pmtu;
 
-	unsigned int congestion_counter;
+	unsigned congestion_counter;
 
-	/* Used for late_connect */
+	/* Used for do_connect for source flows */
 	struct sockaddr *addr;
 	socklen_t addr_len;
 
@@ -127,30 +127,30 @@ struct flow
 		long bytes_read;
 		long bytes_written;
 #endif /* HAVE_UNSIGNED_LONG_LONG_INT */
-		unsigned int request_blocks_read;
-		unsigned int request_blocks_written;
-		unsigned int response_blocks_read;
-		unsigned int response_blocks_written;
+		unsigned request_blocks_read;
+		unsigned request_blocks_written;
+		unsigned response_blocks_read;
+		unsigned response_blocks_written;
 
 		/* TODO Create an array for IAT / RTT and delay */
 
-		/** Minimum interarrival time */
+		/** Minimum interarrival time. */
 		double iat_min;
-		/** Maximum interarrival time */
+		/** Maximum interarrival time. */
 		double iat_max;
-		/** Accumulated interarrival time */
+		/** Accumulated interarrival time. */
 		double iat_sum;
-		/** Minimum one-way delay */
+		/** Minimum one-way delay. */
 		double delay_min;
-		/** Maximum one-way delay */
+		/** Maximum one-way delay. */
 		double delay_max;
-		/** Accumulated one-way delay */
+		/** Accumulated one-way delay. */
 		double delay_sum;
-		/** Minimum round-trip time */
+		/** Minimum round-trip time. */
 		double rtt_min;
-		/** Maximum round-trip time */
+		/** Maximum round-trip time. */
 		double rtt_max;
-		/** Accumulated round-trip time */
+		/** Accumulated round-trip time. */
 		double rtt_sum;
 
 		int has_tcp_info;
@@ -175,6 +175,7 @@ struct flow
 #define REQUEST_START_FLOWS 2
 #define REQUEST_STOP_FLOW 3
 #define REQUEST_GET_STATUS 4
+#define REQUEST_GET_UUID 5
 struct request
 {
 	char type;
@@ -230,6 +231,19 @@ struct request_stop_flow
 	int flow_id;
 };
 
+/**
+ * @brief structure for getting the UUID.
+ *
+ * Structure is used to get the UUID from the daemon and send back the UUID
+ * in request format
+ */
+struct request_get_uuid
+{
+	struct request r;     /**< Daemon thread process the request r. */
+
+	char server_uuid[38]; /**< UUID from the daemon. */
+};
+
 struct request_get_status
 {
 	struct request r;
@@ -238,27 +252,17 @@ struct request_get_status
 	int num_flows;
 };
 
-/** Information about the daemons XMLrpc server */
-struct fg_rpc_server
-{
-	/** Environment used by the Abyss Server */
-	xmlrpc_env env;
-	/** Parameters of the XMLrpc Server */
-	xmlrpc_server_abyss_parms parms;
-};
-
 pthread_t daemon_thread;
 
 /* Through this pipe we wakeup the thread from select */
 extern int daemon_pipe[2];
 
 extern char started;
-extern char dumping;
 extern pthread_mutex_t mutex;
 extern struct linked_list flows;
 extern struct report* reports;
 extern struct report* reports_last;
-extern unsigned int pending_reports;
+extern unsigned pending_reports;
 
 /* Gets 50 reports. There may be more pending but there's a limit on how
  * large a reply can get */
@@ -274,4 +278,19 @@ void flow_error(struct flow *flow, const char *fmt, ...);
 void request_error(struct request *request, const char *fmt, ...);
 int set_flow_tcp_options(struct flow *flow);
 
+/** Dispatch a request to daemon loop.
+ * Is called by the rpc server to feed in requests to the daemon. */
+ int dispatch_request(struct request *request, int type);
+ 
+/**
+ * To generate daemon UUID
+ *
+ * Generate the daemon UUID and convert the UUID to a string data.
+ * UUID is generated by daemon only once and stored in the global variable.
+ * The daemon return the same UUID for all the flows it maintaining.
+ * This UUID is taken as a reference to identify the daemon in the controller.
+ *
+ * @param[in,out] uuid_str daemons UUID
+ */
+ void get_uuid_string(char *uuid_str);
 #endif /* _DAEMON_H_ */
