@@ -32,13 +32,29 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <pcap.h>
-#include <pthread.h>
-
 #include "daemon.h"
 
-void fg_pcap_init(void);
-void fg_pcap_go(struct flow *);
-void fg_pcap_cleanup(void* arg);
+/**
+ * Initialize flowgrind's pcap library.
+ *
+ * This method fills internal structures on which other methods of this library
+ * depend.  It is therefore crucial to call it before any call to other methods
+ * of this library.
+ *
+ * @return return 0 for success, or -1 for failure
+ */
+int fg_pcap_init(void);
+
+/**
+ * Start a tcpdump to capture traffic of the provided flow.
+ *
+ * If the flow was not configured for tcp dumping or dumping is already in
+ * progress the method will do nothing and return immediately. Otherwise the
+ * method blocks until the actual capturing starts. In case an error occurs a
+ * log message is created.
+ *
+ * @param[in] flow the flow whose traffic should be captured
+ */
+void fg_pcap_go(struct flow *flow);
 
 #endif /* _FG_PCAP_H_ */
